"""Naive Tester

Usage:
-----

    $ tester [script] [test_case_folder]

script - a script to test
test_case_folder - a folder containing pairs of .in and .out files,
where
    .in holds [script] arguments
    .out holds expected output

Contact:
-------

More information is available at:
- https://github.com/FilippSolovev/naive-tester

Version:
-------

- naive-tester v1.2.1
"""
import sys
import logging

import tester.config
from tester.io import check_files_existence, get_files
from tester.io import do_files_comply, load_files
from tester.runner import run_jobs

logger = logging.getLogger(__name__)


def main():

    args = [arg for arg in sys.argv[1:] if not arg.startswith('-')]
    opts = [opt for opt in sys.argv[1:] if opt.startswith('-')]

    if len(args) > 1:
        script_name = args[0]
        dir_name = args[1]
    else:
        sys.exit()

    if '-v' in opts:
        tester.config.verbose_output_flg = True

    check_files_existence(script_name, 'Script given does not exist')
    check_files_existence(dir_name, 'Directory given does not exist')

    argument_files = get_files(dir_name, '.in')
    output_files = get_files(dir_name, '.out')
    if not do_files_comply(argument_files, output_files):
        logger.error('Input and output files do not comply')
        sys.exit()

    jobs_arguments = load_files(argument_files)
    reference_outputs = load_files(output_files)

    jobs = zip([script_name for _ in jobs_arguments],
               jobs_arguments)

    jobs_with_outputs = zip(jobs, reference_outputs)

    run_jobs(jobs_with_outputs)


if __name__ == '__main__':
    main()



"""Run an external script and assess its output."""
import contextlib
import logging
import subprocess
import sys
import time

import tester.config


logger = logging.getLogger(__name__)


@contextlib.contextmanager
def timer():
    """Time the execution of a context block.

    Yields:
    ------
        None
    """
    start = time.time()
    yield
    end = time.time()
    logger.info(f'Elapsed time: {(end - start):.2f}s')


def run_job(job):
    """Run a script with it's arguments.

    Parameters
    ----------
    job : a tuple of script's name and its arguments.

    Returns
    --------
    Standard out of a script in a string form.

    """
    script, args = job
    output = subprocess.run(
        [sys.executable] + [script] + [arg for arg in args],
        stdout=subprocess.PIPE,
        universal_newlines=True).stdout.strip('\n')
    return output


def assert_job(job, reference_output):
    """Check if script's output coincides with the reference output."""
    script, args = job
    with timer():
        try:
            reference_output = reference_output[0]
            actual_output = run_job(job)
            assert actual_output == reference_output
            if tester.config.verbose_output_flg:
                logger.info(f'Successfully run the {script}. With argument(s) '
                            f'{args} got the output of {reference_output}.')
            else:
                logger.info(f'Successfully run the {script}.')
        except AssertionError:
            if tester.config.verbose_output_flg:
                logger.error(
                    f'Failed run of the {script} with {args}. '
                    f'The output should be {reference_output}, '
                    f'but got {actual_output}.')
            else:
                logger.error(f'Run of the {script} failed.')


def run_jobs(jobs_with_outputs):
    """Sequentially run scripts."""
    for job_with_output in jobs_with_outputs:
        assert_job(*job_with_output)
